Set Implicit Arguments.
Unset Strict Implicit.



Require Export updateA. 
Require Export infinite. 
Require Export gzloc.

(** we could put this in update_for_gz.v **)
Module Coarse_Cat.
Export GZ_Localization.
Export Infinite.

Definition null_arrow a b := Arrow.create a b emptyset.

Lemma source_null_arrow : forall a b, source (null_arrow a b) = a.
Proof.
ir. 
uf null_arrow. rww Arrow.source_create.
Qed.

Lemma target_null_arrow : forall a b, target (null_arrow a b) = b.
Proof.
ir. 
uf null_arrow. rww Arrow.target_create.
Qed.

Lemma arrow_null_arrow : forall a b, 
arrow (null_arrow a b) = emptyset.
Proof.
ir. 
uf null_arrow. rww Arrow.arrow_create.
Qed.

Definition na_comp u v := null_arrow (source v) (target u).

Lemma source_na_comp : forall u v, source (na_comp u v) = source v.
Proof.
ir. 
uf na_comp. rww source_null_arrow.
Qed.

Lemma target_na_comp : forall u v, target (na_comp u v) = target u.
Proof.
ir. uf na_comp. rww target_null_arrow.
Qed.

Definition is_null_arrow u := u = null_arrow (source  u) (target u).

Lemma is_null_arrow_null_arrow : forall a b,
is_null_arrow (null_arrow a b).
Proof.
ir. 
uhg. 
rw source_null_arrow; rww target_null_arrow.
Qed.

Lemma is_null_arrow_na_comp : forall u v,
is_null_arrow (na_comp u v).
Proof.
ir. uf na_comp. ap is_null_arrow_null_arrow.
Qed.

Lemma null_arrow_extensionality : forall a b,
is_null_arrow a -> is_null_arrow b ->
source a = source b -> target a = target b -> a = b.
Proof.
ir.
uh H; uh H0. rw H. rw H0. 
rw H1; rww H2. 
Qed.

Lemma na_comp_assoc : forall a b c,
na_comp (na_comp a b) c = na_comp a (na_comp b c).
Proof.
ir. 
ap null_arrow_extensionality.
ap is_null_arrow_na_comp. 
ap is_null_arrow_na_comp. 
rw source_na_comp. rw source_na_comp. rww source_na_comp.
rw target_na_comp; rw target_na_comp; rww target_na_comp.
Qed.

Lemma na_comp_null_arrow : forall a b c d,
na_comp (null_arrow a b) (null_arrow c d) =
null_arrow c b.
Proof.
ir.
ap null_arrow_extensionality.
ap is_null_arrow_na_comp.
ap  is_null_arrow_null_arrow.
rw source_na_comp. rw source_null_arrow. rww source_null_arrow.
rw target_na_comp. rw target_null_arrow. rww target_null_arrow.
Qed.


Lemma na_left_id : forall a u,
is_null_arrow a -> 
u = null_arrow (target a) (target a) ->
na_comp u a = a.
Proof.
ir.
ap null_arrow_extensionality.
ap is_null_arrow_na_comp.
am. 
rww source_na_comp. 
rww target_na_comp. rw H0. rww target_null_arrow.
Qed.

Lemma na_right_id : forall a u,
is_null_arrow a -> 
u = null_arrow (source a) (source a) ->
na_comp a u = a.
Proof.
ir.
ap null_arrow_extensionality.
ap is_null_arrow_na_comp.
am. 
rww source_na_comp. rw H0. rww source_null_arrow.
rww target_na_comp. 
Qed.

 
Definition coarse_arrows z :=
Image.create (product z z) 
(fun x => null_arrow (pr1 x) (pr2 x)).

Lemma inc_coarse_arrows : forall u z,
inc u (coarse_arrows z) = 
(is_null_arrow u & inc (source u) z & inc (target u) z).
Proof.
ir. 
ap iff_eq; ir.
ufi coarse_arrows H.
rwi Image.inc_rw H. 
nin H.
ee.
wr H0; ap is_null_arrow_null_arrow.
wr H0. rw source_null_arrow.
rwi inc_product H; ee; am. 
wr H0. rw target_null_arrow.
rwi inc_product H; ee; am. 

uf coarse_arrows.
rw Image.inc_rw. 
sh (pair (source u) (target u)).
ee.
rw inc_product. 
ee. ap pair_is_pair.
rw pr1_pair. am. rww pr2_pair. 
rw pr1_pair. rw pr2_pair. uh H; sy; am. 
Qed.

Definition coarse_cat z :=
Category.Notations.create z (coarse_arrows z) 
na_comp (fun x => null_arrow x x) emptyset.

Lemma is_ob_coarse_cat : forall x z, 
is_ob (coarse_cat z) x = inc x z.
Proof.
ir. 
uf coarse_cat. rww is_ob_create. 
Qed.

Lemma is_mor_coarse_cat : forall u z, 
is_mor (coarse_cat z) u  = 
(is_null_arrow u & inc (source u) z & inc (target u) z).
Proof.
ir. 
uf coarse_cat.
rw is_mor_create. rww inc_coarse_arrows. 
Qed.

Lemma comp_coarse_cat1 : forall u v z,
is_mor (coarse_cat z) u -> is_mor (coarse_cat z) v ->
source u = target v ->
comp (coarse_cat z) u v = na_comp u v.
Proof.
ir. 
uf coarse_cat. 
rwi is_mor_coarse_cat H.
rwi is_mor_coarse_cat H0.

rww comp_create.  

rw inc_coarse_arrows. ee; try am. 
rw inc_coarse_arrows. ee; try am. 
Qed.

Lemma id_coarse_cat : forall x z,
inc x z -> id (coarse_cat z) x = null_arrow x x.
Proof.
ir. 
uf coarse_cat. rww id_create.
Qed.

Lemma coarse_cat_axioms : forall z,
Category.axioms (coarse_cat z). 
Proof.
ir. 
uf coarse_cat. 
ap Category.create_axioms. 
uhg; ee; ir. 
ap iff_eq; ir. 
uhg. ee. am. 
rw inc_coarse_arrows. ee. 
ap is_null_arrow_null_arrow. 
rww source_null_arrow. rww target_null_arrow.
rww source_null_arrow. rww target_null_arrow. 
uh H; ee; am. 

ap iff_eq; ir. 
rwi inc_coarse_arrows H; ee. 
uhg; ee. rw inc_coarse_arrows; ee; am. 
am. am. 
uh H; rw H. 
rw source_null_arrow. rww na_comp_null_arrow.
uh H; rw H. 
rw target_null_arrow. rww na_comp_null_arrow.
uh H; rw H. 
uf null_arrow. rww Arrow.create_like.
uh H; ee; am. 

cp H; cp H0. 
rwi inc_coarse_arrows H; rwi inc_coarse_arrows H0; ee.
uhg; ee. am. am. am. 
rw inc_coarse_arrows. ee.
ap is_null_arrow_na_comp. rww source_na_comp. 
rww target_na_comp. 
rww source_na_comp. rww target_na_comp. 
rww na_comp_assoc. 
Qed.

Lemma ob_coarse_cat : forall x z,
ob (coarse_cat z) x = inc x z.
Proof.
ir.
ap iff_eq; ir. 
uh H. ee.
rwi is_ob_coarse_cat H0; am. 
ap is_ob_ob. ap coarse_cat_axioms. 
rww is_ob_coarse_cat. 
Qed.

Lemma  mor_coarse_cat : forall u z,
mor (coarse_cat z) u = 
(is_null_arrow u & inc (source u) z & inc (target u) z).
Proof.
ir. 
transitivity (is_mor (coarse_cat z) u).
ap iff_eq; ir.
app mor_is_mor. 
app is_mor_mor. ap coarse_cat_axioms. 
rww is_mor_coarse_cat. 
Qed.

Lemma comp_coarse_cat : forall u v z,
mor (coarse_cat z) u -> mor (coarse_cat z) v ->
source u = target v ->
comp (coarse_cat z) u v = na_comp u v.
Proof.
ir. 
rww comp_coarse_cat1. 
app mor_is_mor.  
app mor_is_mor.  
Qed.

End Coarse_Cat.

Module Lf_Counterexample.
Export Coarse_Cat. 
Export Left_Fraction_Category.

Inductive cx_ob : E -> Prop :=
is0 : cx_ob (R 0) |
is1 : cx_ob (R 1) |
is2 : cx_ob (R 2).

Ltac is_cx_ob :=
match goal with 
| |- cx_ob (R 0) => ap is0
| |- cx_ob (R 1) => ap is1
| |- cx_ob (R 2) => ap is2
| _ => fail end.


Lemma cx_ob_rw : forall x,
cx_ob x = inc x (R 3).
Proof.
ir. 
ap iff_eq; ir.
nin H; rw inc_lt; om. 

rwi R_S_tack_on H; rwi tack_on_inc H. nin H.  
rwi R_S_tack_on H; rwi tack_on_inc H. nin H. 
rwi R_S_tack_on H; rwi tack_on_inc H. nin H. 
rwi zero_emptyset H. 
nin H. nin x0. 
rw H; is_cx_ob. rw H; is_cx_ob. rw H; is_cx_ob.
Qed.

Notation na00 := (null_arrow (R 0) (R 0)).
Notation na11 := (null_arrow (R 1) (R 1)).
Notation na22 := (null_arrow (R 2) (R 2)).

Notation na01 := (null_arrow (R 0) (R 1)).
Notation na02 := (null_arrow (R 0) (R 2)).

Notation na12 := (null_arrow (R 1) (R 2)).
Notation na21 := (null_arrow (R 2) (R 1)).

Inductive cx_mor : E -> Prop :=
is00 :  cx_mor na00 |
is11 : cx_mor na11 |
is22 : cx_mor na22 |
is01 : cx_mor na01 |
is02 : cx_mor na02 |
is12 : cx_mor na12 |
is21 : cx_mor na21 .

Ltac is_cx_mor :=
match goal with 
| |- cx_mor na00 => ap is00
| |- cx_mor na11 => ap is11 
| |- cx_mor na22 => ap is22 
| |- cx_mor na01 => ap is01 
| |- cx_mor na02 => ap is02 
| |- cx_mor na12 => ap is12 
| |- cx_mor na21 => ap is21 
| _ => fail end. 
 


Lemma cx_mor_mor : forall u, cx_mor u -> 
mor (coarse_cat (R 3)) u.
Proof.
ir. 
rw mor_coarse_cat. ee. 
nin H; ap is_null_arrow_null_arrow. 
wr cx_ob_rw. 
nin H; rw source_null_arrow; is_cx_ob.  
wr cx_ob_rw.
nin H; rw target_null_arrow; is_cx_ob. 
Qed.



Definition cx := subcategory (coarse_cat (R 3))
cx_ob cx_mor.

Ltac R_nat_discriminate :=
match goal with 
| id1 : R ?X1 = R ?X2 |- _ => 
assert (discr : X1 = X2); [app R_inj | discriminate discr]
| _ => fail
end.


Lemma cx_property : subcategory_property (coarse_cat (R 3))
cx_ob cx_mor.
Proof.
ir. 
uhg; ee; ir. 
ap coarse_cat_axioms. rw comp_coarse_cat.

nin H2; nin H3; 
try (rwi source_null_arrow H1; rwi target_null_arrow H1; 
R_nat_discriminate);  try 
(rw na_comp_null_arrow; is_cx_mor).
app cx_mor_mor. app cx_mor_mor. am. 
rw id_coarse_cat. 
nin H0; is_cx_mor. 
wrr cx_ob_rw. 
nin H0; rw source_null_arrow; is_cx_ob. 
nin H0; rw target_null_arrow; is_cx_ob. 
Qed. 

Lemma cx_axioms : Category.axioms cx.
Proof.
ir. 
unfold cx. 
ap subcategory_axioms. 
ap cx_property. 
Qed. 

Lemma ob_cx : forall x, ob cx x = cx_ob x.
Proof.
ir.
unfold cx. rw ob_subcategory.
ap iff_eq; ir.
ee; am. ee.
rw ob_coarse_cat. rwi cx_ob_rw H. am. am. 
ap cx_property. 
Qed. 

Lemma mor_cx : forall u, mor cx u = cx_mor u.
Proof.
ir.
unfold cx. rw mor_subcategory.
ap iff_eq; ir.
ee; am. ee.
app cx_mor_mor. am. 
ap cx_property. 
Qed. 

Inductive in_cx_sys : E -> Prop :=
sys00 : in_cx_sys na00 |
sys11 : in_cx_sys na11 |
sys22 : in_cx_sys na22 |
sys01 : in_cx_sys na01 |
sys02 : in_cx_sys na02.

Ltac is_cx_sys := 
match goal with 
| |- in_cx_sys na00 => ap sys00
| |- in_cx_sys na11 => ap sys11
| |- in_cx_sys na22 => ap sys22
| |- in_cx_sys na01 => ap sys01
| |- in_cx_sys na02 => ap sys02
| _ => fail end. 


Definition cx_sys := Z (morphisms cx) in_cx_sys.

Lemma inc_cx_sys : forall u,
inc u cx_sys = in_cx_sys u.
Proof.
ir.
ap iff_eq; ir.
ufi cx_sys H. Ztac. 
uf cx_sys.
ap Z_inc.
ap mor_is_mor.
rw mor_cx. nin H; is_cx_mor.
am. 
Qed.

Lemma source_null_arrow_eq : forall a b c d,
null_arrow a b = null_arrow c d -> a = c.
Proof.
ir.
transitivity (source (null_arrow a b)).
rww source_null_arrow.
rw H. rww source_null_arrow.
Qed. 

Lemma target_null_arrow_eq : forall a b c d,
null_arrow a b = null_arrow c d -> b = d.
Proof.
ir.
transitivity (target (null_arrow a b)).
rww target_null_arrow.
rw H. rww target_null_arrow.
Qed. 

Lemma cx_sys_rw : forall u,
inc u cx_sys = (in_cx_sys u &
mor cx u & cx_mor u & ~u=na12 & ~u=na21).
Proof.
ir.
ap iff_eq; ir.
rwi inc_cx_sys H.
ee; try am. 
rw mor_cx.  
nin H; is_cx_mor. 
nin H; is_cx_mor. 
uhg; ir.
nin H; 
cp (source_null_arrow_eq H0);
cp (target_null_arrow_eq H0);
R_nat_discriminate. 

uhg; ir.
nin H; 
cp (source_null_arrow_eq H0);
cp (target_null_arrow_eq H0);
R_nat_discriminate. 

rw inc_cx_sys;
ee; am. 
Qed.

Lemma comp_cx : forall u v,
mor cx u -> mor cx v -> source u = target v ->
comp cx u v = na_comp u v.
Proof.
ir.
unfold cx. 
rw comp_subcategory. rw comp_coarse_cat.
tv. ap cx_mor_mor.
rwi mor_cx H; am. 
ap cx_mor_mor.
rwi mor_cx H0; am. am. 
ap cx_property. 
ap cx_mor_mor.
rwi mor_cx H; am. 
ap cx_mor_mor.
rwi mor_cx H0; am. am. 
rwi mor_cx H; am. 
rwi mor_cx H0; am. 
Qed.


Lemma id_cx : forall x,
ob cx x -> id cx x = null_arrow x x.
Proof.
ir. 
unfold cx. 
rw id_subcategory. 
rw id_coarse_cat. tv. 
rwi ob_cx H. wrr cx_ob_rw. 
ap cx_property. 
rw ob_coarse_cat. wrr cx_ob_rw. 
wrr ob_cx. wrr ob_cx. 
Qed.

Lemma localizing_system_cx_sys: 
localizing_system cx cx_sys.
Proof.
uhg; ee. ap cx_axioms. 
ir. 
rwi cx_sys_rw H. ee; am.  
Qed. 

Lemma multiplicative_system_cx_sys: 
multiplicative_system cx cx_sys.
Proof.
uhg; ee. 
ap localizing_system_cx_sys.
ir. 
rwi cx_sys_rw H; rwi cx_sys_rw H0; ee.
rww comp_cx. 
rw inc_cx_sys.
nin H7; nin H3; 
try (rwi source_null_arrow H1; rwi target_null_arrow H1;
R_nat_discriminate);
try (rw na_comp_null_arrow;
is_cx_sys).

assert False.
app H5. elim H3.

assert False.
app H4. elim H3.

assert False.
app H8. elim H3.

assert False.
app H9. elim H3.
Qed. 

Lemma has_left_fractions_cx_sys : 
has_left_fractions cx cx_sys.
Proof.
(** in order to avoid [lf_symbol] and other specific
notations we rewrite as per the lemma in [gzloc.v]. **)
rw has_left_fractions_rw.
ee.
ap multiplicative_system_cx_sys.

ir.
rw id_cx. rwi ob_cx H. 
rw inc_cx_sys. 
nin H; is_cx_sys. am. 

ir.
apply by_cases with 
((r = id cx (source r)) \/ 
(g = id cx (source g))). 


(** the case [r = id cx (source r)] **)
ir; nin H2.
sh g. 
sh (id cx (target g)).

ee. 
am. 
rw id_cx. 
rw inc_cx_sys. 
rwi mor_cx H0. nin H0; 
rw target_null_arrow; 
is_cx_sys. 
rww ob_target. 
rww target_id. 
rww ob_target. rw H2. 

rww target_id. 
sy; am. rww ob_source. 
rwi cx_sys_rw H; ee; am. 
rww source_id. rww ob_target. 
rww left_id. 
rw H2. rw H1. rww right_id. 
rww ob_source. rww ob_target. 

(** the case [g = id cx (source g)] **)
rwi cx_sys_rw H. 
ee. 
sh (id cx (target r)).
sh r. 
ee.
ap mor_id. rww ob_target. rww inc_cx_sys. 
rww target_id. rww ob_target. 
rww source_id. rww ob_target. 
rw H2. rww target_id. 
rww ob_source. 
rww left_id. rw H2. wr H1.
rww right_id. rww ob_source. rww ob_target. 

(** now we try to eliminate most other cases **)
ir. 

apply by_cases with (r=na01 \/ r = na02); ir.

apply by_cases with (g = na01 \/ g = na02); ir.

nin H3; nin H4; rw H3; rw H4.

(** the case [r = na01] and [g = na01] **)
sh (null_arrow (R 1) (R 1)).
sh (null_arrow (R 1) (R 1)).
ee.
rw mor_cx. is_cx_mor. 
rw inc_cx_sys. is_cx_sys. tv. 
rw source_null_arrow; rww target_null_arrow.
rw source_null_arrow; rww target_null_arrow.
tv. 

(** the case [r = na01] and [g = na02] **)
sh (null_arrow (R 1) (R 2)).
sh (null_arrow (R 2) (R 2)).
ee.
rw mor_cx. is_cx_mor. 
rw inc_cx_sys. is_cx_sys. tv. 
rw target_null_arrow; rww target_null_arrow.
rw source_null_arrow; rww target_null_arrow.
rw source_null_arrow; rww target_null_arrow.
rw comp_cx. rw comp_cx. 
rw na_comp_null_arrow.
rww na_comp_null_arrow. 
rw mor_cx; is_cx_mor. 
rw mor_cx; is_cx_mor. 
rw source_null_arrow; rww target_null_arrow.
rw mor_cx; is_cx_mor. 
rw mor_cx; is_cx_mor. 
rw source_null_arrow; rww target_null_arrow.

(** the case [r = na02] and [g = na01] **)
sh (null_arrow (R 2) (R 1)).
sh (null_arrow (R 1) (R 1)).
ee.
rw mor_cx. is_cx_mor. 
rw inc_cx_sys. is_cx_sys. tv. 
rw target_null_arrow; rww target_null_arrow.
rw source_null_arrow; rww target_null_arrow.
rw source_null_arrow; rww target_null_arrow.
rw comp_cx. rw comp_cx. 
rw na_comp_null_arrow.
rww na_comp_null_arrow. 
rw mor_cx; is_cx_mor. 
rw mor_cx; is_cx_mor. 
rw source_null_arrow; rww target_null_arrow.
rw mor_cx; is_cx_mor. 
rw mor_cx; is_cx_mor. 
rw source_null_arrow; rww target_null_arrow.

(** the case [r = na02] and [g = na02] **)
sh (null_arrow (R 2) (R 2)).
sh (null_arrow (R 2) (R 2)).
ee.
rw mor_cx. is_cx_mor. 
rw inc_cx_sys. is_cx_sys. tv. 
rw source_null_arrow; rww target_null_arrow.
rw source_null_arrow; rww target_null_arrow.
tv. 

(** now we show that we have covered all the possible cases. **)


rwi id_cx H2. 
rwi id_cx H2. 

rwi inc_cx_sys H. 
rwi mor_cx H0.
nin H; nin H0;
try (rwi source_null_arrow H1; rwi source_null_arrow H1; 
R_nat_discriminate);
try (rwi source_null_arrow H2;
try (rwi source_null_arrow H2);
nin H2; solve [app or_introl | app or_intror]).
nin H4. 
app or_introl. 
nin H4. app or_intror. 
nin H4. app or_introl. 
nin H4; app or_intror. 
rww ob_source. rwi cx_sys_rw H; ee. 
rww ob_source. 

(** unfortunately because of a glitch in how we did the
[by_cases] at the start we have to basically
recopy the above. **)
rwi id_cx H2. 
rwi id_cx H2. 

rwi inc_cx_sys H. 
rwi mor_cx H0.
nin H; nin H0;
try (rwi source_null_arrow H1; rwi source_null_arrow H1; 
R_nat_discriminate);
try (rwi source_null_arrow H2;
try (rwi source_null_arrow H2);
nin H2; solve [app or_introl | app or_intror]).
nin H3. 
app or_introl. 
nin H3. app or_introl. 
nin H3. app or_intror. 
nin H3; app or_intror. 
rww ob_source. rwi cx_sys_rw H; ee. 
rww ob_source. 

(** we have now finished condition (c) **)

(** condition (d) is easy since arrows are determined
by their source and target **)

ir.
assert (source r = source t).
rww H3. 
assert  (target r = target t).
transitivity (target (comp cx r v)).
rww target_comp. rwi cx_sys_rw H; ee; am. 
rw H4. 
rww target_comp. rwi cx_sys_rw H; ee; am. 

assert (mor (coarse_cat (R 3)) r).
ap cx_mor_mor. wrr mor_cx. 
assert (mor (coarse_cat (R 3)) t).
ap cx_mor_mor. wrr mor_cx. 

rwi mor_coarse_cat H7. 
rwi mor_coarse_cat H8. ee.

assert (r = t).
uh H7; uh H8.
rw H7; rw H8.
rw H5; rw H6; tv. 

sh (id cx (target r)).
ee.
rw id_cx. 
rw inc_cx_sys.

rwi mor_cx H0. nin H0; rw target_null_arrow; is_cx_sys. 
rww ob_target. 
rww source_id. rww ob_target. 
rw H6. 
rww source_id. rww ob_target. 
rww left_id. 
rw H6. rww left_id. 
rww ob_target. rww ob_target. 
Qed. 

(** the two [lf_symbol]'s we want to compare are as follows. **)

Definition lf110 := lf_symbol na11 na01.

Definition lf120 := lf_symbol na12 na02.

Lemma lff110 : lf_forward lf110 = na11. 
Proof.
uf lf110.
rww lf_forward_lf_symbol. 
Qed.

Lemma lfb110 : lf_backward lf110 = na01. 
Proof.
uf lf110.
rww lf_backward_lf_symbol. 
Qed.

Lemma lff120 : lf_forward lf120 = na12. 
Proof.
uf lf120.
rww lf_forward_lf_symbol. 
Qed.

Lemma lfb120 : lf_backward lf120 = na02. 
Proof.
uf lf120.
rww lf_backward_lf_symbol. 
Qed.

Lemma is_lf_symbol_lf110 : is_lf_symbol cx cx_sys lf110.
Proof.
ir.
uhg; ee.
uf lf110. 
ap lf_symbol_like_lf_symbol.
uhg; ee.
ap localizing_system_cx_sys. rw lff110.
rw mor_cx.
is_cx_mor.
rw lfb110. rw inc_cx_sys.
is_cx_sys.
rw lff110. rw lfb110. 
rw target_null_arrow. rww target_null_arrow. 
Qed.

Lemma is_lf_symbol_lf120 : is_lf_symbol cx cx_sys lf120.
Proof.
ir.
uhg; ee.
uf lf120. 
ap lf_symbol_like_lf_symbol.
uhg; ee.
ap localizing_system_cx_sys. rw lff120.
rw mor_cx.
is_cx_mor.
rw lfb120. rw inc_cx_sys.
is_cx_sys.
rw lff120. rw lfb120. 
rw target_null_arrow. rww target_null_arrow. 
Qed.

(** Recall that there are two notions, [lf_beyond] and
[lf_under]. The first is existence of an intermediate arrow,
and the second stronger condition is existence of an
intermediate arrow in the localizing system. Our aim is
to show that lf110 and lf120 cannot be joined by
a sequence of relations of the form [lf_under].
We concretize this by showing that [lf_under lf110 u -> u = lf110]
and [lf_under lf120 u -> u = lf120].
On the other hand we will see that [lf_beyond lf110 lf120]
(meaning that they project to equal arrows in the localization).
**)

Lemma lfv110 : lf_vertex lf110 = R 1.
Proof.
uf lf_vertex. rw lfb110. rww target_null_arrow.
Qed. 

Lemma lfv120 : lf_vertex lf120 = R 2.
Proof.
uf lf_vertex. rw lfb120. rww target_null_arrow.
Qed. 


Lemma under_lf110_same : forall u,
lf_under cx cx_sys lf110 u -> u = lf110.
Proof.
ir.
uh H; ee.
nin H4; ee. 
rwi lfv110 H5. 

assert (x = na11).
rwi inc_cx_sys H4. 
nin H4; 
rwi source_null_arrow H5; try R_nat_discriminate. 
tv. rwi H7 H6.
wr H6.
uf lf_extend.
rw lff110. 
rw lfb110. 
rw comp_cx; try (rw mor_cx; is_cx_mor). 
rw comp_cx; try (rw mor_cx; is_cx_mor). 
rw na_comp_null_arrow. 
rw na_comp_null_arrow. tv. 
rw source_null_arrow. 
rww target_null_arrow. 
rw source_null_arrow; rww target_null_arrow. 
Qed.


Lemma under_lf120_same : forall u,
lf_under cx cx_sys lf120 u -> u = lf120.
Proof.
ir.
uh H; ee.
nin H4; ee. 
rwi lfv120 H5. 

assert (x = na22).
rwi inc_cx_sys H4. 
nin H4; 
rwi source_null_arrow H5; try R_nat_discriminate. 
tv. rwi H7 H6.
wr H6.
uf lf_extend.
rw lff120. 
rw lfb120. 
rw comp_cx; try (rw mor_cx; is_cx_mor). 
rw comp_cx; try (rw mor_cx; is_cx_mor). 
rw na_comp_null_arrow. 
rw na_comp_null_arrow. tv. 
rw source_null_arrow. 
rww target_null_arrow. 
rw source_null_arrow; rww target_null_arrow. 
Qed.

Lemma lf110_different_lf120 : ~lf110 = lf120.
Proof.
uhg; ir. 
assert (lf_vertex lf110 = lf_vertex lf120).
rww H. rwi lfv110 H0. rwi lfv120 H0. 
R_nat_discriminate. 
Qed. 

Lemma lf_beyond_lf110_lf120 : lf_beyond cx cx_sys lf110 lf120.
Proof.
uhg; ee. 
ap has_left_fractions_cx_sys. 
ap is_lf_symbol_lf110. 
ap is_lf_symbol_lf120. 
uf lf110. uf lf120. 
rw source_lf_symbol. rw source_lf_symbol. 
rw source_null_arrow; rww source_null_arrow. 
uf lf110; uf lf120.
rw target_lf_symbol. sy; rw target_lf_symbol.
rw source_null_arrow; rww source_null_arrow.
sh (null_arrow (R 1) (R 2)).
ee.
rw mor_cx. is_cx_mor. 
rw lfv110. rww source_null_arrow. 

uf lf_extend. 
rw lff110. rw lfb110. 
rw comp_cx; try (rw mor_cx; is_cx_mor). 
rw comp_cx; try (rw mor_cx; is_cx_mor). 
rw na_comp_null_arrow. 
rw na_comp_null_arrow. tv. 
rw source_null_arrow. 
rww target_null_arrow. 
rw source_null_arrow; rww target_null_arrow. 
Qed.


Lemma beyond_not_under_counterexample : 
exists a, exists s, exists u, exists v,
(has_left_fractions a s &
is_lf_symbol a s u & 
is_lf_symbol a s v &
lf_beyond a s u v & 
~(exists w, (lf_under a s u w & lf_under a s v w))).
Proof.
ir. 
apply ex_intro with cx. 
sh cx_sys. 
sh lf110. 
sh lf120.
ee.
ap has_left_fractions_cx_sys. 
ap is_lf_symbol_lf110. 
ap is_lf_symbol_lf120. 
ap lf_beyond_lf110_lf120.

uhg; ir.
nin H. 
ee. 
cp (under_lf110_same H). 
cp (under_lf120_same H0).
ap lf110_different_lf120. wrr H1. 
Qed.





End Lf_Counterexample.
